"use strict";

/* exported logger */

var ObjectWithListeners = require('../ObjectWithListeners');
var PanelControlElementType = require('./PanelControlElementType');

// noinspection JSUnusedLocalSymbols
var logger = require('../logger');

/**
 *
 * @param params
 * @constructor
 *
 * @extends ObjectWithListeners
 */
function AbstractGuiElement(params) {
  ObjectWithListeners.call(this, params);

  var self = this;

  self.setElement(params.element);
  self.setProject(params.project);
  self.setMap(params.customMap);
  self.setConfiguration(params.configuration);

  self._controlElements = [];

}

AbstractGuiElement.prototype = Object.create(ObjectWithListeners.prototype);
AbstractGuiElement.prototype.constructor = AbstractGuiElement;

/**
 *
 * @param {CustomMap} map
 */
AbstractGuiElement.prototype.setMap = function (map) {
  if (map === undefined) {
    throw new Error("map must be defined");
  }
  this._map = map;
};

/**
 *
 * @returns {CustomMap}
 */
AbstractGuiElement.prototype.getMap = function () {
  return this._map;
};

/**
 *
 * @param {HTMLElement} element
 */
AbstractGuiElement.prototype.setElement = function (element) {
  if (element === undefined || element === null) {
    throw new Error("DOM Element must be defined");
  }
  this._element = element;
};

/**
 *
 * @returns {HTMLElement}
 */
AbstractGuiElement.prototype.getElement = function () {
  return this._element;
};

/**
 *
 * @param {Project} project
 */
AbstractGuiElement.prototype.setProject = function (project) {
  this._project = project;
};

/**
 *
 * @returns {Project}
 */
AbstractGuiElement.prototype.getProject = function () {
  if (this._project === undefined || this._project === null) {
    return this.getMap().getProject();
  } else {
    return this._project;
  }
};


AbstractGuiElement.prototype.setControlElement = function (type, element) {
  if (type === null || type === undefined) {
    throw new Error("Unknown control element type");
  }
  if (PanelControlElementType[type] === undefined) {
    throw new Error("Unknown control element type: " + type);
  }

  this._controlElements[type] = element;
};

AbstractGuiElement.prototype.getControlElement = function (type) {
  if (type === null || type === undefined) {
    throw new Error("Unknown control element type");
  }
  if (PanelControlElementType[type] === undefined) {
    throw new Error("Unknown control element type: " + type);
  }

  return this._controlElements[type];
};

AbstractGuiElement.prototype.downloadFile = function (url) {
  this._downloadFile = url;
  window.open(url, '_blank');
};

AbstractGuiElement.prototype.getLastDownloadUrl = function () {
  return this._downloadFile;
};

/**
 *
 * @param {Configuration} configuration
 */
AbstractGuiElement.prototype.setConfiguration = function (configuration) {
  this._configuration = configuration;
};

/**
 *
 * @returns {Configuration}
 */
AbstractGuiElement.prototype.getConfiguration = function () {
  return this._configuration;
};

/**
 *
 * @param {string} [params.content]
 * @param {string} params.title
 * @param {boolean} [params.input]
 * @returns {Promise}
 */
AbstractGuiElement.prototype.askConfirmRemoval = function (params) {
  return new Promise(function (resolve) {
    var html;
    var content = '';
    if (params.content) {
      content = params.content;
    }
    if (params.input) {
      html = '<form><input type="text" style="z-index:10000" name="reason-value"><br></form>';
    } else {
      html = '<form><span>' + content + '</span><input type="text" style="z-index:10000;visibility: hidden" name="reason-value"><br></form>';
    }
    $(html).dialog({
      modal: true,
      title: params.title,
      close: function () {
        $(this).dialog('destroy').remove();
        resolve({status: false});
      },
      buttons: {
        'OK': function () {
          var reason = $('input[name="reason-value"]').val();
          $(this).dialog('destroy').remove();
          resolve({reason: reason, status: true});
        },
        'Cancel': function () {
          $(this).dialog('destroy').remove();
          resolve({status: false});
        }
      }
    });
  })
};


module.exports = AbstractGuiElement;
